"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodePoolWorker = void 0;
const node_util_1 = require("node:util");
const microservice_1 = require("@jfrog-pipelines/microservice");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const modules_1 = require("./modules");
const Marshaller_1 = require("./modules/node/Marshaller");
const CommandFactory_1 = require("./modules/CommandFactory");
const ORPHAN_NODES_COLLECTION_INTERVAL_THREE_MIN_IN_MS = 1800000;
class NodePoolWorker {
    constructor(details, context) {
        this.details = details;
        this.context = context;
        this.logger = logger_1.LoggerFactory.getLogger('nodePoolWorker');
        this.nodePoolService = new modules_1.NodePoolService();
        this.systemProperties = configuration_1.ConfigFactory.config;
    }
    async process(message) {
        try {
            const payload = message?.payload;
            if (payload) {
                if (payload.marshaller) {
                    await Marshaller_1.Marshaller.callMarshaller(payload.marshaller);
                    return;
                }
                if (payload.nodePool) {
                    this.logger.info('Processing nodePool', (0, node_util_1.inspect)(payload.nodePool));
                    await this.processNodePool(payload.nodePool);
                    return;
                }
                if (payload.command) {
                    this.logger.info(`Processing command`, (0, node_util_1.inspect)(payload.command));
                    const command = CommandFactory_1.CommandFactory.getCommand(payload);
                    const handler = CommandFactory_1.CommandFactory.getHandler(command);
                    await handler.handle(command);
                    return;
                }
            }
            else {
                throw new Error(`Invalid payload received ${(0, node_util_1.inspect)(payload)}`);
            }
        }
        catch (error) {
            this.logger.error((0, node_util_1.inspect)(error));
        }
    }
    async startCollectingOrphanNodes() {
        const publisher = new amqp_client_1.AmqpPublisher(amqp_client_1.AmqpFactory.getClient(), this.systemProperties.amqpExchange, amqp_client_1.publisherOptions, this.logger);
        const collector = new modules_1.NodeCollector(publisher);
        return this.collectOrphanNodes(collector);
    }
    async processNodePool(payload) {
        const publisher = new amqp_client_1.AmqpPublisher(amqp_client_1.AmqpFactory.getClient(), this.systemProperties.amqpExchange, amqp_client_1.publisherOptions, this.logger);
        const poolID = payload.nodePoolId;
        const systemInstanceId = this.context.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER)?.systemServiceInstancesId;
        this.logger.trace(`Processing for node pool with id: ${poolID} with system service id: ${systemInstanceId}`);
        const lockService = new modules_1.LockService(systemInstanceId);
        const lockName = [...this.details.name, '-nodePools-', ...String(poolID)].join('');
        try {
            const onDemandNodeStartTime = Date.now();
            const nodePool = await this.nodePoolService.getNodePool(poolID);
            if (!nodePool) {
                this.logger.warn(`NodePool id ${poolID} is not dynamic nodepool`);
                return;
            }
            this.logger.trace(`Successfully fetched node pool with id: ${poolID} and node pool name is ${nodePool.name}`);
            if (!nodePool.isOnDemand) {
                this.logger.info(`Node Pool is not on demand, wont continue with execution`);
                return;
            }
            const collector = new modules_1.NodeCollector(publisher);
            const provisioner = new modules_1.NodeProvisioner(publisher);
            const lock = await lockService.obtainLock({
                name: lockName,
                lockTimeoutInMs: modules_1.LockTimeoutInMs
            });
            if (!lock) {
                this.logger.info(`Failed to obtain lock for node pool id ${poolID}`);
                return;
            }
            this.logger.trace(`Successfully obtained lock for node pool ${nodePool.name} with lock name: ${lockName}`);
            const onDemandNodeEndTime = Date.now();
            const onDemandNodeTimeDiff = new Date(onDemandNodeEndTime - onDemandNodeStartTime);
            const requiredNodes = await this.getRequiredNodes(poolID);
            const onDemandNodeData = {
                domain: 'nodepool',
                eventType: 'onDemandNodes',
                durationMillis: onDemandNodeTimeDiff.valueOf(),
                count: requiredNodes,
                nodePoolName: nodePool.name
            };
            logger_1.LoggerFactory.logEvent(onDemandNodeData);
            this.logger.trace(`Required Node count is ${requiredNodes}`);
            if (nodePool.isBeta) {
                this.logger.trace('Beta node pool, will not de-provision node');
                await provisioner.provisionBeta(nodePool);
            }
            else {
                await provisioner.provision(nodePool, requiredNodes);
                await collector.deProvision(nodePool, requiredNodes);
            }
        }
        catch (error) {
            this.logger.error(`Failed to process node pool id ${poolID} with err ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            await lockService.releaseLock(lockName);
            this.logger.trace(`Lock was released successfully with name: ${lockName}`);
        }
    }
    async getRequiredNodes(poolID) {
        const features = new modules_1.FeatureAwareFactory();
        return features.getRequiredNodes(poolID);
    }
    async collectOrphanNodes(collector) {
        this.logger.debug('Collecting orphan nodes started');
        await collector.collectOrphanNodes();
        this.logger.debug(`Orphan nodes were successfully collected. Will be recollected in next ${ORPHAN_NODES_COLLECTION_INTERVAL_THREE_MIN_IN_MS / 1000 / 60} mins`);
        setTimeout(() => {
            this.collectOrphanNodes(collector);
        }, ORPHAN_NODES_COLLECTION_INTERVAL_THREE_MIN_IN_MS);
    }
}
exports.NodePoolWorker = NodePoolWorker;
//# sourceMappingURL=NodePoolWorker.js.map