"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const amqp_client_1 = require("@jfrog-pipelines/amqp-client");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const configuration_1 = require("@jfrog-pipelines/configuration");
const NodeCollector_1 = require("../NodeCollector");
const NodePoolService_1 = require("../nodePool/NodePoolService");
const NodeService_1 = require("../node/NodeService");
jest.mock('@jfrog-pipelines/amqp-client');
jest.mock('@jfrog/access-nodejs-client');
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            configure: jest.fn(),
            logEvent: jest.fn(),
            getEventLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            })
        }
    };
});
const client = new amqp_client_1.AmqpClient('', '', { amqpUrl: '' }, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publisher = new amqp_client_1.AmqpPublisher(client, '', amqp_client_1.publisherOptions, logger_1.LoggerFactory.getLogger('NodeServiceTest'));
const publishMessageMock = jest.spyOn(amqp_client_1.AmqpPublisher.prototype, 'publishMessage').mockResolvedValueOnce({});
jest.spyOn(internal_clients_1.AccessClientFactory, 'getClient').mockReturnValue({
    getNodeAccessToken: jest.fn().mockReturnValue('token')
});
jest.spyOn(configuration_1.ConfigFactory, 'config', 'get').mockImplementation(() => {
    return { rtRouterJoinKey: 'abcd1234' };
});
describe('Test NodeCollector #deProvision', () => {
    const MS = 1000;
    const nodeCollector = new NodeCollector_1.NodeCollector(publisher);
    const nodePool = { id: 1, name: 'testpool', projectId: 1, isCacheEnabled: true, isOnDemand: true };
    const getNodePoolMock = jest.spyOn(NodePoolService_1.NodePoolService.prototype, 'getNodePool').mockResolvedValueOnce(nodePool);
    const oneMin = 60 * MS;
    const sixteenMins = 16 * oneMin;
    const sixteenMinsBefore = new Date(Date.now() - sixteenMins);
    afterEach(() => {
        publishMessageMock.mockClear();
        getNodePoolMock.mockClear();
    });
    const STATUS_UPDATED_TIMEOUT = 15 * 60000;
    const PROVISIONED_DATE_TIME = 15 * 24 * 60 * 60000;
    const nodes = [
        { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
        {
            friendlyName: 'test2',
            projectId: 1,
            nodePoolId: 1,
            statusCode: NodeService_1.NodeStatus.success,
            statusLastUpdatedAt: new Date(Date.now() - STATUS_UPDATED_TIMEOUT),
            provisionedAt: new Date(Date.now() - PROVISIONED_DATE_TIME)
        }
    ];
    it('should delete success nodes when statusLastUpdatedAt > nodeIdleInterval', async () => {
        const oneMin = 60 * MS;
        const oneMinuteBefore = new Date(Date.now() - oneMin);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: oneMinuteBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: false,
            nodeIdleIntervalInMins: 1
        };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped };
        const eventLoggerSpy = jest.spyOn(logger_1.LoggerFactory, 'logEvent');
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(1);
        expect(eventLoggerSpy).toHaveBeenCalledWith(expect.objectContaining({
            count: 1,
            domain: 'nodepool',
            eventType: 'onNodeReclaimed',
            nodePoolName: 'testpool'
        }));
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should cache success nodes when statusLastUpdatedAt > nodeIdleInterval when isCacheEnabled is true', async () => {
        const oneMin = 60 * MS;
        const oneMinuteBefore = new Date(Date.now() - oneMin);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                id: 1,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: oneMinuteBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                id: 2,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1
        };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped };
        const applyNodeToCachedStateSpy = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToCachedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToCachedStateSpy).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeToCachedStateSpy.mockClear();
    });
    it('should  not change deleted nodes based on nodeStopSetting', async () => {
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopSetting: '' };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete cached node based on nodeStopIntervalDays', async () => {
        const oneDay = 24 * 60 * 60 * MS;
        const twoDays = 2 * oneDay;
        const oneDayBefore = new Date(Date.now() - oneDay);
        const twoDayBefore = new Date(Date.now() - twoDays);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                provisionedAt: oneDayBefore,
                statusLastUpdatedAt: oneDayBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                provisionedAt: twoDayBefore,
                statusLastUpdatedAt: oneDayBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1,
            cacheSettingsPropertyBag: { nodeStopIntervalDays: 1, nodeStopSetting: 'nodeStopIntervalDays' }
        };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(2);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should not delete cached node which is provisioned today', async () => {
        const oneMin = 60 * MS;
        const sixteenMins = 16 * oneMin;
        const sixteenMinsBefore = new Date(Date.now() - sixteenMins);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1,
            cacheSettingsPropertyBag: { nodeStopIntervalDays: 1, nodeStopSetting: 'nodeStopIntervalDays' }
        };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(0);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete cached node which is configured based on nodeStopDayOfWeek', async () => {
        jest.useFakeTimers();
        const fakeDate = new Date();
        fakeDate.setUTCFullYear(2023, 1, 5);
        fakeDate.setHours(0, 0, 0);
        jest.setSystemTime(fakeDate);
        const oneMin = 60 * MS;
        const sixteenMins = 16 * oneMin;
        const sixteenMinsBefore = new Date(Date.now() - sixteenMins);
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1,
            cacheSettingsPropertyBag: { nodeStopSetting: 'nodeStopDayOfWeek', nodeStopDayOfWeek: 'sunday' }
        };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(2);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
        jest.useRealTimers();
    });
    it('should  not delete node based on nodeProvisionedAt', async () => {
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopIntervalDays: 20, nodeStopSetting: 'nodeStopIntervalDays' };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should not deprovision node which are waiting', async () => {
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.waiting,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: new Date()
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.waiting,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const np = { ...nodePool, nodeIdleIntervalInMins: 2, isCacheEnabled: true };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const applyNodeToCachedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToCachedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToCachedStateMock).toHaveBeenCalledTimes(0);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(0);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should  delete node based on nodeProvisionedAt', async () => {
        const nodes = [
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.cached },
            {
                friendlyName: 'test2',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.success,
                statusLastUpdatedAt: new Date(),
                provisionedAt: new Date()
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getRunningNodes').mockResolvedValueOnce(nodes);
        nodePool.cacheSettingsPropertyBag = { nodeStopIntervalDays: 20, nodeStopSetting: 'nodeStopIntervalDays' };
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete failure nodes', async () => {
        const nodes = [{ friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.failure }];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete waiting nodes which have crossed cuttoffTime', async () => {
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1
        };
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.waiting,
                statusLastUpdatedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete having stopRequested = true', async () => {
        const np = {
            ...nodePool,
            isCacheEnabled: true,
            nodeIdleIntervalInMins: 1
        };
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.waiting,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.success,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.failure,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.stopped,
                stopRequested: true
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                stopRequested: true
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(5);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete stopped nodes', async () => {
        const nodes = [{ friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped }];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete cached nodes', async () => {
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.cached,
                statusLastUpdatedAt: sixteenMinsBefore,
                provisionedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped };
        const np = {
            ...nodePool,
            isCacheEnabled: false,
            nodeIdleIntervalInMins: 1
        };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should not delete processing nodes', async () => {
        const nodes = [
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.processing,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            },
            {
                friendlyName: 'test1',
                projectId: 1,
                nodePoolId: 1,
                statusCode: NodeService_1.NodeStatus.processing,
                provisionedAt: sixteenMinsBefore,
                statusLastUpdatedAt: sixteenMinsBefore
            }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(0);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete deleted nodes', async () => {
        const nodes = [{ friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.deleted }];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const np = {
            ...nodePool,
            isCacheEnabled: false,
            nodeIdleIntervalInMins: 1
        };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(np, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(1);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should delete failure, stopped, deleted nodes', async () => {
        const nodes = [
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.stopped },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.failure },
            { friendlyName: 'test1', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.deleted }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getAllNodes').mockResolvedValueOnce(nodes);
        const node = { friendlyName: 'test', projectId: 1, nodePoolId: 1, statusCode: NodeService_1.NodeStatus.waiting };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const deProvisionResponse = await nodeCollector.deProvision(nodePool, 0);
        await expect(deProvisionResponse).toBe(undefined);
        expect(applyNodeToStoppedStateMock).toHaveBeenCalledTimes(3);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
    it('should  collect orphan nodes', async () => {
        const nodes = [
            { friendlyName: 'test1', projectId: 1, nodePoolId: null, statusCode: NodeService_1.NodeStatus.waiting },
            { friendlyName: 'test2', projectId: 1, nodePoolId: null, statusCode: NodeService_1.NodeStatus.success },
            { friendlyName: 'test3', projectId: 1, nodePoolId: null, statusCode: NodeService_1.NodeStatus.cached },
            { friendlyName: 'test4', projectId: 1, nodePoolId: null, statusCode: NodeService_1.NodeStatus.processing }
        ];
        const getRunningNodesMock = jest.spyOn(NodeService_1.NodeService.prototype, 'getOrphanNodes').mockResolvedValueOnce(nodes);
        const node = { statusCode: NodeService_1.NodeStatus.stopped };
        const applyNodeToStoppedStateMock = jest
            .spyOn(NodeService_1.NodeService.prototype, 'applyNodeToStoppedState')
            .mockResolvedValueOnce(node);
        const orphanNodesCollected = await nodeCollector.collectOrphanNodes();
        await expect(orphanNodesCollected).toBe(4);
        getRunningNodesMock.mockClear();
        applyNodeToStoppedStateMock.mockClear();
    });
});
//# sourceMappingURL=NodeCollector.test.js.map